define([ 'emvc/core/Model', 'emvc/core/View', 'emvc/core/Controller', 'emvc/components/rabbitFarm', 'emvc/components/rabbitMachine' ], function (Model, View, Controller, rabbitFarm, rabbitMachine) {

    /**
    * App
    *
    * @class App
    * @constructor
    */
    function App() {

        /**
        * Main model
        *
        * @property model
        * @extends Model
        * @type Object
        */
        this.model = new Model({

            rabbits: 0
    
        });

        /**
        * Main controller
        *
        * @property controller
        * @extends Controller
        * @type Object
        */
        this.controller = new Controller({

            incrementValue: function(model) {
                model.rabbits++;
                Events.publish("/Whatever", [model.rabbits]);
            }

        });

        /**
        * Rabbit Farm View
        *
        * @property table
        * @extends View
        * @uses table
        * @type Object
        */
        this.rabbitFarmView = new View(this.model, this.controller, {

            uses: [ 'emvc/components/rabbitFarm' ],
 
            id: "rabbit-farm",

            // This could be the template. It's not being used for the time being ^__^
            tpl: "templates/farm.handlebars",

            // Event handlers
            events: {
                "/Whatever/change": "render" 
            },

            // Render
            render: function() {

                var self = this;

                function addRabbit(rabbitImage) {
                    var rabbit = document.createElement("img");
                    rabbit.src = rabbitImage;
                    self.el.appendChild(rabbit);
                }

                this.el = document.getElementById(this.id);
                this.el.innerHTML = "";

                var html = document.createElement("div");
                html.innerHTML = '<h3>Rabbit Farm</h3><p id="numeration">Number of rabbits: <span>' + this.model.rabbits + '</span></p>';
                this.el.appendChild(html);


                for (var i=0; i<this.model.rabbits; i++) {
                    addRabbit("img/rabbit.png");
                };

                this.applyDomListeners();

            }

        });

        /**
        * Rabbit Machine view
        *
        * @property filterPanel
        * @extends View
        * @uses lateralPanel
        * @type Object
        */
        this.rabbitMachineView = new View( this.model, this.controller, {

            uses: [ 'emvc/components/rabbitMachine' ],

            id: "rabbit-machine",

            // Event handlers
            events: {
                "#circle click": "clickHandler" 
            },

            // Just a property to check that this property is added to the view
            // when er are extending it
            rabbitSpecification: {
                color: "white",
                size: "small",
                ears: "long"
            },

            // Render
            render: function() {

                this.el = document.getElementById(this.id);
                this.el.innerHTML = "";

                var html = document.createElement("div");
                html.innerHTML = '<div id="circle">Click Me</div><p class="instructions"><em>Click the button to add rabbits to the Rabbit Farm</em><br /><span class="arrow">&rarr;</span></p><p><em>The view that contains this button calls the controller when the button is clicked. The controller modifies the model in which the number of rabbits resides. The rabbit farm view is connected to the model, and when the model changes an event is fired that updates the rabbit farm view\'s number of rabbits.</em></p>';
                this.el.appendChild(html);

                this.applyDomListeners();

            },

            // UI functions
            clickHandler: function(e) {
                this.controller.incrementValue(this.model);
                e.preventDefault();
            }

        })

    };

    return App;

});